/*********************************************************************
 *
 * File:         server.c
 *
 * Description:  NCS users server
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         12-17-96
 *
 *********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>

#include <netdb.h>
#include <unistd.h>

#include "server.h"
#include "global.h"

void messageHandle(int socket, MsgHeader *header, char *msg, int len);
int readServerMsg(int socket);

ServerInfo hServer;

FILE *log;

void nic()
{
}


int readAll(int fd, void* buffer, size_t len)
{
  int st;
  
  do
    {
      st = read( fd, buffer, len );
      if( st > 0 )
        {
	  len -= st;
	  buffer = ( char* )buffer + st;
        } 
      else
        {
	  if( st < 0 )
            {
	      perror( "readAll" );
	      return -1;
            }
	  return 0;
        }
    }
  while( len );

  return 1;
}


int readMsgHeader(int sock, MsgHeader *header)
{
  char buf[MSG_HEADER_LENGTH];
  int err;

  PRINTF("Server: reading header\n");
  err = readAll(sock, buf, MSG_HEADER_LENGTH);
  if(err == 0)
    {
      fprintf(stderr, "Server: connection broken\n");
      return err;
    }
  if(err < 0)
    {
      fprintf(stderr, "Server: reading error\n");
      return err;
    }

  header->tag = buf[0];
  header->session = buf[1];
  header->sender.handle = buf[2];
  strcpy(header->sender.user, &buf[3]);
  header->sender.host = *(int *)&buf[23];
  header->receiver.handle = buf[27];
  strcpy(header->receiver.user, &buf[28]);
  header->receiver.host = *(int *)&buf[48];
  header->dataLength = *(int *)&buf[52];

  return err;
}

void quitServer()
{
  fflush(stdout);
  close(hServer.socket);
  fclose(log);
  exit(0);
}

/* Server initialization */
void initServer()
{
  signal(SIGINT, quitServer);

  if ((hServer.socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
      fprintf(stderr, "Server: can't open socket\n");
      exit(-1);
    }

  bzero(&hServer.addr, sizeof(hServer.addr));
  hServer.addr.sin_family = AF_INET;
  hServer.addr.sin_addr.s_addr = htonl(INADDR_ANY);
  hServer.addr.sin_port        = htons(SERVER_PORT);

  if(bind(hServer.socket, &hServer.addr, sizeof(hServer.addr)) < 0)
    {
      fprintf(stderr, "Server: can't bind socket\n");
      exit(-1);
    }

  listen(hServer.socket, 5);

  /*#ifdef _DEBUG*/
  log = fopen("server.log", "w");
  /*#endif*/
}


/**************************************************************************
 *
 * Main socket reading procedure
 *
 **************************************************************************/
void runServer()
{
  struct sockaddr_in remote_addr;
  int remote_addr_len;
  int sock;
  fd_set set;
  struct timeval timeout;

  remote_addr_len = sizeof(remote_addr);
  timeout.tv_sec = 1;
  while(1)
    {
      FD_ZERO(&set);
      FD_SET(hServer.socket, &set);
      if(select(hServer.socket+1, &set, 0, 0, &timeout) > 0)
	{
	  if(FD_ISSET(hServer.socket, &set))
	    {
	      sock = accept(hServer.socket, &remote_addr, &remote_addr_len);
	      readServerMsg(sock);
	    }
	  fflush(stdout);
	  fflush(stderr);
	}
    }
}

int readServerMsg(int sock)
{
  char *msg;
  MsgHeader header;
  int err;

  PRINTF("Server: read message\n");

  err = readMsgHeader(sock, &header);
  if(err == 0)
    {
      PRINTF("Engine: connection broken\n");
      close(sock);
      return;
    }
  if(err < 0)
    {
      fprintf(stderr, "Engine: data reading error\n");
      close(sock);
      return;
    }

  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(sock, msg, header.dataLength);
      if(err == 0)
	{
	  PRINTF("Engine: connection broken\n");
	  close(sock);
	  return;
	}
      if(err < 0)
	{
	  fprintf(stderr, "Engine: data reading error\n");
	  close(sock);
	  return;
	}
    }
  else
    msg = NULL;
  
  messageHandle(sock, &header, msg, header.dataLength);
  
  free(msg);
  close(sock);
}

/*********************************************************************
 *
 * Incoming messages handling procedures
 *
 *********************************************************************/
void messageHandle(int sock, MsgHeader *header, char *msg, int len)
{
  switch(header->tag)
    {
    case reqServerAddUser:
      fprintf(log, "Request: add user %s@%d\n", header->sender.user, header->sender.host);
      add_user(&header->sender);
      break;

    case reqServerDelUser:
      fprintf(log, "Request: delete user %s@%d\n", header->sender.user, header->sender.host);
      del_user(&header->sender);
      break;

    case reqServerUsers:
      fprintf(log, "Request: users from %s@%d\n", header->sender.user, header->sender.host);
      send_users(sock, &header->sender);
      break;

    default:
      fprintf(stderr, "Wrong server tag %d\n", header->tag);
    }
}




/***********************************************************************
 *
 *                            MAIN
 *
 **********************************************************************/
void main(void)
{
  initServer();
  runServer();
}










